@@ -5,9 +5,11 @@ from collections import defaultdict
 import numpy as np
 
 from glue.core import Subset
+from glue.utils import nanmin, nanmax
 from glue.external.echo import (delay_callback, CallbackProperty,
                                 HasCallbackProperties, CallbackList)
 from glue.core.state import saver, loader
+from glue.core.component_id import PixelComponentID
 
 __all__ = ['State', 'StateAttributeCacheHelper',
            'StateAttributeLimitsHelper', 'StateAttributeSingleValueHelper']
@@ -254,6 +256,10 @@ class StateAttributeLimitsHelper(StateAttributeCacheHelper):
     percentile_subset : int
         How many points to use at most for the percentile calculation (using all
         values is highly inefficient and not needed)
+    margin : float
+        Whether to add a margin to the range of values determined. This should be
+        given as a floating point value that will be multiplied by the range of
+        values to give the margin to add to the limits.
     lower, upper : str
         The fields for the lower/upper levels
     percentile : ``QComboBox`` instance, optional
@@ -279,11 +285,13 @@ class StateAttributeLimitsHelper(StateAttributeCacheHelper):
     values_names = ('lower', 'upper')
     modifiers_names = ('log', 'percentile')
 
-    def __init__(self, state, attribute, percentile_subset=10000, cache=None, **kwargs):
+    def __init__(self, state, attribute, percentile_subset=10000, margin=0, cache=None, **kwargs):
 
         super(StateAttributeLimitsHelper, self).__init__(state, attribute, cache=cache, **kwargs)
 
+        self.margin = margin
         self.percentile_subset = percentile_subset
+        self.subset_indices = None
 
         if self.attribute is not None:
 
@@ -316,12 +324,23 @@ class StateAttributeLimitsHelper(StateAttributeCacheHelper):
 
         else:
 
+            # Shortcut if the component ID is a pixel component ID
+            if isinstance(self.component_id, PixelComponentID) and percentile == 100 and not log:
+                lower = -0.5
+                upper = self.data.shape[self.component_id.axis] - 0.5
+                self.set(lower=lower, upper=upper, percentile=percentile, log=log)
+                return
+
             exclude = (100 - percentile) / 2.
 
             data_values = self.data_values
 
             if data_values.size > self.percentile_subset:
-                data_values = np.random.choice(data_values.ravel(), self.percentile_subset)
+                if self.subset_indices is None or self.subset_indices[0] != data_values.size:
+                    self.subset_indices = (data_values.size,
+                                           np.random.randint(0, data_values.size,
+                                                             self.percentile_subset))
+                data_values = data_values.ravel()[self.subset_indices[1]]
 
             if log:
                 data_values = data_values[data_values > 0]
@@ -329,11 +348,22 @@ class StateAttributeLimitsHelper(StateAttributeCacheHelper):
                     self.set(lower=0.1, upper=1, percentile=percentile, log=log)
                     return
 
-            try:
-                lower = np.nanpercentile(data_values, exclude)
-                upper = np.nanpercentile(data_values, 100 - exclude)
-            except AttributeError:  # Numpy < 1.9
-                data_values = data_values[~np.isnan(data_values)]
+            # NOTE: we can't use np.nanmin/np.nanmax or nanpercentile below as
+            # they don't exclude inf/-inf
+            if data_values.dtype.kind != 'M':
+                data_values = data_values[np.isfinite(data_values)]
+
+            if percentile == 100:
+
+                if data_values.dtype.kind == 'M':
+                    lower = data_values.min()
+                    upper = data_values.max()
+                else:
+                    lower = np.min(data_values)
+                    upper = np.max(data_values)
+
+            else:
+
                 lower = np.percentile(data_values, exclude)
                 upper = np.percentile(data_values, 100 - exclude)
 
@@ -341,6 +371,15 @@ class StateAttributeLimitsHelper(StateAttributeCacheHelper):
                 lower = np.floor(lower - 0.5) + 0.5
                 upper = np.ceil(upper + 0.5) - 0.5
 
+            if log:
+                value_range = np.log10(upper / lower)
+                lower /= 10.**(value_range * self.margin)
+                upper *= 10.**(value_range * self.margin)
+            else:
+                value_range = upper - lower
+                lower -= value_range * self.margin
+                upper += value_range * self.margin
+
             self.set(lower=lower, upper=upper, percentile=percentile, log=log)
 
     def flip_limits(self):
@@ -442,8 +481,13 @@ class StateAttributeHistogramHelper(StateAttributeCacheHelper):
                     n_bin = self._common_n_bin
 
                 values = self.data_values
-                lower = np.nanmin(values)
-                upper = np.nanmax(values)
+
+                if comp.datetime:
+                    lower = values.min()
+                    upper = values.max()
+                else:
+                    lower = nanmin(values)
+                    upper = nanmax(values)
 
             self.set(lower=lower, upper=upper, n_bin=n_bin)
 
