@@ -1,5 +1,6 @@
 from __future__ import absolute_import, division, print_function
 
+import uuid
 import numbers
 import operator
 
@@ -16,14 +17,15 @@ from glue.core.message import SubsetDeleteMessage, SubsetUpdateMessage
 from glue.core.decorators import memoize
 from glue.core.visual import VisualAttributes
 from glue.config import settings
-from glue.utils import view_shape, broadcast_to
+from glue.utils import view_shape, broadcast_to, floodfill, combine_slices
 
 
 __all__ = ['Subset', 'SubsetState', 'RoiSubsetState', 'CategoricalROISubsetState',
            'RangeSubsetState', 'MultiRangeSubsetState', 'CompositeSubsetState',
            'OrState', 'AndState', 'XorState', 'InvertState', 'MaskSubsetState', 'CategorySubsetState',
            'ElementSubsetState', 'InequalitySubsetState', 'combine_multiple',
-           'CategoricalMultiRangeSubsetState', 'CategoricalROISubsetState2D']
+           'CategoricalMultiRangeSubsetState', 'CategoricalROISubsetState2D',
+           'SliceSubsetState']
 
 
 OPSYM = {operator.ge: '>=', operator.gt: '>',
@@ -74,6 +76,16 @@ class Subset(object):
         self.style.color = color
         self.style.alpha = alpha
 
+        # We assign a UUID which can then be used for example in equations
+        # for derived components - the idea is that this doesn't change over
+        # the life cycle of glue, so it is a more reliable way to refer to
+        # components in strings than using labels
+        self._uuid = str(uuid.uuid4())
+
+    @property
+    def uuid(self):
+        return self._uuid
+
     @property
     def subset_state(self):
         return self._subset_state
@@ -346,7 +358,10 @@ class Subset(object):
         self.subset_state = state
 
     def __del__(self):
-        self.delete()
+        try:
+            self.delete()
+        except Exception:
+            pass
 
     def __setattr__(self, attribute, value):
         object.__setattr__(self, attribute, value)
@@ -541,7 +556,7 @@ class RoiSubsetState(SubsetState):
 
         if (x.ndim == data.ndim and
             self.xatt in data.pixel_component_ids and
-            self.yatt in data.pixel_component_ids):
+                self.yatt in data.pixel_component_ids):
 
             # This is a special case - the ROI is defined in pixel space, so we
             # can apply it to a single slice and then broadcast it to all other
@@ -699,6 +714,7 @@ class CategoricalROISubsetState2D(SubsetState):
     att2 : :class:`~glue.core.component_id.ComponentID`
         The component ID matching the values of the ``categories`` dictionary
     """
+
     def __init__(self, categories, att1, att2):
         self.categories = categories
         self.att1 = att1
@@ -854,6 +870,7 @@ class CompositeSubsetState(SubsetState):
         sym = OPSYM.get(self.op, self.op)
         return "(%s %s %s)" % (self.state1, sym, self.state2)
 
+
 class OrState(CompositeSubsetState):
     op = operator.or_
 
@@ -896,7 +913,8 @@ class MaskSubsetState(SubsetState):
 
     def to_mask(self, data, view=None):
 
-        view = view or slice(None)
+        if view is None:
+            view = slice(None)
 
         # shortcut for data on the same pixel grid
         if data.pixel_component_ids == self.cids:
@@ -921,6 +939,108 @@ class MaskSubsetState(SubsetState):
                    [context.object(c) for c in rec['cids']])
 
 
+class SliceSubsetState(SubsetState):
+    """
+    A subset defined by a slice in an array
+    """
+
+    def __init__(self, reference_data, slices):
+        self.reference_data = reference_data
+        self.slices = slices
+        self._pad_slices()
+
+    def _pad_slices(self):
+        from glue.core.data import Data
+        if isinstance(self.reference_data, Data) and len(self.slices) < self.reference_data.ndim:
+            self.slices = self.slices + [slice(None)] * (self.reference_data.ndim - len(self.slices))
+
+    def copy(self):
+        return SliceSubsetState(self.reference_data, self.slices)
+
+    def to_mask(self, data, view=None):
+
+        if view is None:
+            view = Ellipsis
+        elif isinstance(view, slice) or np.isscalar(view):
+            view = [view]
+
+        # Figure out the shape of the final mask given the requested view
+        shape = view_shape(data.shape, view)
+
+        if data is self.reference_data:
+
+            slices = self.slices
+
+        else:
+
+            # Check if we can transform list of slices to match this dataset
+            order = data.pixel_aligned_data.get(self.reference_data, None)
+
+            if order is None:
+                # We use broadcast_to for minimal memory usage
+                return broadcast_to(False, shape)
+            else:
+                # Reorder slices
+                slices = [self.slices[idx] for idx in order]
+
+        if (isinstance(view, np.ndarray) or
+                (isinstance(view, (tuple, list)) and isinstance(view[0], np.ndarray))):
+            mask = np.zeros(data.shape, dtype=bool)
+            mask[slices] = True
+            return mask[view]
+
+        # The original slices assume the full array, not the array with the view
+        # applied, so we need to now adjust the slices accordingly.
+        if view is Ellipsis:
+            subslices = slices
+        else:
+            subslices = []
+            for i in range(data.ndim):
+                if i >= len(view):
+                    subslices.append(slices[i])
+                elif np.isscalar(view[i]):
+                    beg, end, stp = slices[i].indices(data.shape[i])
+                    if view[i] < beg or view[i] >= end or (view[i] - beg) % stp != 0:
+                        return broadcast_to(False, shape)
+                elif isinstance(view[i], slice):
+                    if view[i].step is not None and view[i].step < 0:
+                        beg, end, step = view[i].indices(data.shape[i])
+                        v = slice(end + 1, beg + 1, -step)
+                    else:
+                        v = view[i]
+                    subslices.append(combine_slices(v, slices[i], data.shape[i]))
+                else:
+                    raise TypeError("Unexpected view item: {0}".format(view[i]))
+
+        # Create mask with final shape
+        mask = np.zeros(shape, dtype=bool)
+        mask[subslices] = True
+
+        return mask
+
+    def to_array(self, data, att):
+        if data is self.reference_data:
+            slices = self.slices
+        else:
+            order = data.pixel_aligned_data.get(self.reference_data, None)
+            if order is None:
+                raise IncompatibleAttribute()
+            slices = [self.slices[idx] for idx in order]
+        return data[att, slices]
+
+    def __gluestate__(self, context):
+        return dict(slices=context.do(self.slices),
+                    reference_data=context.id(self.reference_data))
+
+    @classmethod
+    def __setgluestate__(cls, rec, context):
+        return cls(rec['reference_data'], context.object(rec['slices']))
+
+    def __setgluestate_callback__(self, context):
+        self.reference_data = context.object(self.reference_data)
+        self._pad_slices()
+
+
 class CategorySubsetState(SubsetState):
 
     def __init__(self, attribute, values):
@@ -1067,7 +1187,6 @@ class InequalitySubsetState(SubsetState):
                 else:
                     right = comp.data[view]
 
-
         return self._operator(left, right)
 
     def copy(self):
@@ -1081,6 +1200,120 @@ class InequalitySubsetState(SubsetState):
         return '<%s: %s>' % (self.__class__.__name__, self)
 
 
+class FloodFillSubsetState(MaskSubsetState):
+    """
+    A subset state representing a flood-fill operation, which is computed
+    on-the-fly.
+    """
+
+    # TODO: we need to recompute the mask if the numerical values of the
+    # data changes.
+
+    def __init__(self, data, attribute, start_coords, threshold):
+
+        if len(start_coords) != data.ndim:
+            raise ValueError("start_coords should have as many values as data "
+                             "has dimensions.")
+
+        self.attribute = attribute
+        self.data = data
+        self.start_coords = tuple(start_coords)
+        self.threshold = float(threshold)
+        self.cids = self.data.pixel_component_ids
+
+        self._compute_mask()
+
+    def _compute_mask(self):
+        mask = floodfill(self.data[self.attribute],
+                         self.start_coords, self.threshold)
+        self._mask_cache = (self._hash, mask)
+
+    @property
+    def _hash(self):
+        return self.data, self.attribute, self.start_coords, self.threshold, self.cids
+
+    @property
+    def mask(self):
+        if self._mask_cache[0] != self._hash:
+            self._compute_mask()
+        return self._mask_cache[1]
+
+    def copy(self):
+        return FloodFillSubsetState(self.data, self.attribute, self.start_coords,
+                                    self.threshold)
+
+    def __gluestate__(self, context):
+        # We don't store the data since this would cause a circular reference.
+        # However we can recover the data from the attribute ComponentID.
+        return dict(attribute=context.id(self.attribute),
+                    start_coords=self.start_coords,
+                    threshold=self.threshold)
+
+    @classmethod
+    def __setgluestate__(cls, rec, context):
+        attribute = context.object(rec['attribute'])
+        return cls(attribute.parent, attribute,
+                   context.object(rec['start_coords']),
+                   context.object(rec['threshold']))
+
+
+class RoiSubsetState3d(SubsetState):
+    """Subset state for a roi that implements .contains3d
+    """
+
+    @contract(xatt='isinstance(ComponentID)', yatt='isinstance(ComponentID)', zatt='isinstance(ComponentID)')
+    def __init__(self, xatt=None, yatt=None, zatt=None, roi=None):
+        super(RoiSubsetState3d, self).__init__()
+        self.xatt = xatt
+        self.yatt = yatt
+        self.zatt = zatt
+        self.roi = roi
+
+    @property
+    def attributes(self):
+        return (self.xatt, self.yatt, self.zatt)
+
+    @contract(data='isinstance(Data)', view='array_view')
+    def to_mask(self, data, view=None):
+
+        # TODO: make sure that pixel components don't actually take up much
+        #       memory and are just views
+        x = data[self.xatt, view]
+        y = data[self.yatt, view]
+        z = data[self.zatt, view]
+
+        if self.roi.defined():
+            result = self.roi.contains3d(x, y, z)
+        else:
+            result = np.zeros(x.shape, dtype=bool)
+
+        if result.shape != x.shape:
+            raise ValueError("Unexpected error: boolean mask has incorrect dimensions")
+
+        return result
+
+    def copy(self):
+        result = RoiSubsetState3d()
+        result.xatt = self.xatt
+        result.yatt = self.yatt
+        result.zatt = self.zatt
+        result.roi = self.roi
+        return result
+
+    def __gluestate__(self, context):
+        return dict(xatt=context.id(self.xatt),
+                    yatt=context.id(self.yatt),
+                    zatt=context.id(self.zatt),
+                    roi=context.id(self.roi))
+
+    @classmethod
+    def __setgluestate__(cls, rec, context):
+        return RoiSubsetState3d(context.object(rec['xatt']),
+                                context.object(rec['yatt']),
+                                context.object(rec['zatt']),
+                                context.object(rec['roi']))
+
+
 @contract(subsets='list(isinstance(Subset))', returns=Subset)
 def _combine(subsets, operator):
     state = operator(*[s.subset_state for s in subsets])
