@@ -6,6 +6,7 @@ import os
 from collections import Counter
 
 import pytest
+import numpy as np
 
 from numpy.testing import assert_equal, assert_allclose
 
@@ -14,8 +15,8 @@ from glue.core import HubListener, Data
 from glue.core.roi import XRangeROI
 from glue.core.subset import RangeSubsetState, CategoricalROISubsetState
 from glue import core
+from glue.app.qt import GlueApplication
 from glue.core.component_id import ComponentID
-from glue.core.tests.util import simple_session
 from glue.utils.qt import combo_as_string
 from glue.viewers.matplotlib.qt.tests.test_data_viewer import BaseTestMatplotlibDataViewer
 from glue.core.state import GlueUnSerializer
@@ -38,19 +39,20 @@ class TestHistogramViewer(object):
 
         self.data = Data(label='d1', x=[3.4, 2.3, -1.1, 0.3], y=['a', 'b', 'c', 'a'])
 
-        self.session = simple_session()
+        self.app = GlueApplication()
+        self.session = self.app.session
         self.hub = self.session.hub
 
         self.data_collection = self.session.data_collection
         self.data_collection.append(self.data)
 
-        self.viewer = HistogramViewer(self.session)
-
-        self.data_collection.register_to_hub(self.hub)
-        self.viewer.register_to_hub(self.hub)
+        self.viewer = self.app.new_data_viewer(HistogramViewer)
 
     def teardown_method(self, method):
         self.viewer.close()
+        self.viewer = None
+        self.app.close()
+        self.app = None
 
     def test_basic(self):
 
@@ -59,7 +61,7 @@ class TestHistogramViewer(object):
         # Check defaults when we add data
         self.viewer.add_data(self.data)
 
-        assert combo_as_string(self.viewer.options_widget().ui.combosel_x_att) == 'x:y'
+        assert combo_as_string(self.viewer.options_widget().ui.combosel_x_att) == 'Main components:x:y:Coordinate components:Pixel Axis 0 [x]:World 0'
 
         assert viewer_state.x_att is self.data.id['x']
         assert viewer_state.x_min == -1.1
@@ -98,6 +100,33 @@ class TestHistogramViewer(object):
         assert not viewer_state.x_log
         assert not viewer_state.y_log
 
+    def test_log_labels(self):
+
+        # Regression test to make sure the labels are correctly changed to log
+        # when the x-axis is in log space.
+
+        viewer_state = self.viewer.state
+        data = Data(x=np.logspace(-5, 5, 10000))
+        self.data_collection.append(data)
+
+        self.viewer.add_data(data)
+        viewer_state.x_log = True
+
+        labels = [x.get_text() for x in self.viewer.axes.xaxis.get_ticklabels()]
+
+        # Different Matplotlib versions return slightly different
+        # labels, but the ones below should be present regardless
+        # of Matplotlib version.
+        expected_present = ['$\\mathdefault{10^{-5}}$',
+                            '$\\mathdefault{10^{-3}}$',
+                            '$\\mathdefault{10^{-1}}$',
+                            '$\\mathdefault{10^{1}}$',
+                            '$\\mathdefault{10^{3}}$',
+                            '$\\mathdefault{10^{5}}$']
+
+        for label in expected_present:
+            assert label in labels
+
     def test_flip(self):
 
         viewer_state = self.viewer.state
@@ -114,7 +143,7 @@ class TestHistogramViewer(object):
 
     def test_remove_data(self):
         self.viewer.add_data(self.data)
-        assert combo_as_string(self.viewer.options_widget().ui.combosel_x_att) == 'x:y'
+        assert combo_as_string(self.viewer.options_widget().ui.combosel_x_att) == 'Main components:x:y:Coordinate components:Pixel Axis 0 [x]:World 0'
         self.data_collection.remove(self.data)
         assert combo_as_string(self.viewer.options_widget().ui.combosel_x_att) == ''
 
@@ -128,7 +157,7 @@ class TestHistogramViewer(object):
         self.viewer.add_data(self.data)
         self.data.add_component([3, 4, 1, 2], 'z')
         assert self.viewer.state.x_att is self.data.id['x']
-        assert combo_as_string(self.viewer.options_widget().ui.combosel_x_att) == 'x:y:z'
+        assert combo_as_string(self.viewer.options_widget().ui.combosel_x_att) == 'Main components:x:y:z:Coordinate components:Pixel Axis 0 [x]:World 0'
 
     def test_nonnumeric_first_component(self):
         # regression test for #208. Shouldn't complain if
@@ -404,11 +433,11 @@ class TestHistogramViewer(object):
         # was selected
 
         self.viewer.add_data(self.data)
-        self.viewer.state.x_att = self.data.components[0]
+        self.viewer.state.x_att = self.data.id['x']
         test = ComponentID('test')
         self.data.update_id(self.viewer.state.x_att, test)
         assert self.viewer.state.x_att is test
-        assert combo_as_string(self.viewer.options_widget().ui.combosel_x_att) == 'test:y'
+        assert combo_as_string(self.viewer.options_widget().ui.combosel_x_att) == 'Main components:test:y:Coordinate components:Pixel Axis 0 [x]:World 0'
 
     def test_nbin_override_persists_over_numerical_attribute_change(self):
 
@@ -512,7 +541,7 @@ class TestHistogramViewer(object):
         # messages regarding subset updates (this occurred when multiple)
         # datasets were present.
 
-        layer_tree = LayerTreeWidget()
+        layer_tree = LayerTreeWidget(session=self.session)
         layer_tree.set_checkable(False)
         layer_tree.setup(self.data_collection)
         layer_tree.bind_selection_to_edit_subset()
@@ -550,3 +579,48 @@ class TestHistogramViewer(object):
 
         for subset in client.count:
             assert client.count[subset] == 1
+
+    def test_datetime64_support(self, tmpdir):
+
+        self.data.add_component(np.array([100, 200, 300, 400], dtype='M8[D]'), 't1')
+        self.viewer.add_data(self.data)
+        self.viewer.state.x_att = self.data.id['t1']
+
+        # Matplotlib deals with dates by converting them to the number of days
+        # since 01-01-0001, so we can check that the limits are correctly
+        # converted (and not 100 to 400)
+        assert self.viewer.axes.get_xlim() == (719263.0, 719563.0)
+
+        # Apply an ROI selection in plotting coordinates
+        roi = XRangeROI(719313, 719513)
+        self.viewer.apply_roi(roi)
+
+        # Check that the two middle elements are selected
+        assert_equal(self.data.subsets[0].to_mask(), [0, 1, 1, 0])
+
+        # Make sure that the Qt labels look ok
+        options = self.viewer.options_widget().ui
+        assert options.valuetext_x_min.text() == '1970-04-11'
+        assert options.valuetext_x_max.text() == '1971-02-05'
+
+        # Make sure that we can set the xmin/xmax to a string date
+        assert_equal(self.viewer.state.x_min, np.datetime64('1970-04-11', 'D'))
+        options.valuetext_x_min.setText('1970-04-14')
+        options.valuetext_x_min.editingFinished.emit()
+        assert self.viewer.axes.get_xlim() == (719266.0, 719563.0)
+        assert_equal(self.viewer.state.x_min, np.datetime64('1970-04-14', 'D'))
+
+        # Make sure that everything works fine after saving/reloading
+        filename = tmpdir.join('test_datetime64.glu').strpath
+        self.session.application.save_session(filename)
+        with open(filename, 'r') as f:
+            session = f.read()
+        state = GlueUnSerializer.loads(session)
+        ga = state.object('__main__')
+        viewer = ga.viewers[0][0]
+        options = viewer.options_widget().ui
+
+        assert_equal(self.viewer.state.x_min, np.datetime64('1970-04-14', 'D'))
+
+        assert options.valuetext_x_min.text() == '1970-04-14'
+        assert options.valuetext_x_max.text() == '1971-02-05'
