@@ -7,6 +7,7 @@ from qtpy import QtWidgets
 from glue.external.echo.qt import autoconnect_callbacks_to_qt
 from glue.utils.qt import load_ui, fix_tab_widget_fontsize
 from glue.viewers.image.qt.slice_widget import MultiSliceWidgetHelper
+from glue.viewers.matplotlib.state import MatplotlibDataViewerState
 
 __all__ = ['ImageOptionsWidget']
 
@@ -22,16 +23,18 @@ class ImageOptionsWidget(QtWidgets.QWidget):
 
         fix_tab_widget_fontsize(self.ui.tab_widget)
 
-        self.ui.combodata_aspect.addItem("Square Pixels", userData='equal')
-        self.ui.combodata_aspect.addItem("Automatic", userData='auto')
-        self.ui.combodata_aspect.setCurrentIndex(0)
-
-        self.ui.combotext_color_mode.addItem("Colormaps")
-        self.ui.combotext_color_mode.addItem("One color per layer")
-
         autoconnect_callbacks_to_qt(viewer_state, self.ui)
 
         self.viewer_state = viewer_state
 
         self.slice_helper = MultiSliceWidgetHelper(viewer_state=self.viewer_state,
                                                    layout=self.ui.layout_slices)
+
+        self.session = session
+        self.ui.axes_editor.button_apply_all.clicked.connect(self._apply_all_viewers)
+
+    def _apply_all_viewers(self):
+        for tab in self.session.application.viewers:
+            for viewer in tab:
+                if isinstance(viewer.state, MatplotlibDataViewerState):
+                    viewer.state.update_axes_settings_from(self.viewer_state)
