@@ -8,15 +8,10 @@ from matplotlib.figure import Figure
 
 from qtpy import QtCore, QtGui, QtWidgets
 from qtpy.QtCore import Qt
-from qtpy import PYQT5
 from glue.config import settings
 
-if PYQT5:
-    from matplotlib.backends.backend_qt5 import FigureManagerQT as FigureManager
-    from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
-else:
-    from matplotlib.backends.backend_qt4 import FigureManagerQT as FigureManager
-    from matplotlib.backends.backend_qt4agg import FigureCanvasQTAgg as FigureCanvas
+from matplotlib.backends.backend_qt5 import FigureManagerQT as FigureManager
+from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
 
 # We want to ignore warnings about left==right and bottom==top since these are
 # not critical and the default behavior makes sense.
@@ -124,6 +119,10 @@ class MplCanvas(FigureCanvas):
         self._draw_count += 1
         return super(MplCanvas, self).draw(*args, **kwargs)
 
+    def keyPressEvent(self, event):
+        event.setAccepted(False)
+        super(MplCanvas, self).keyPressEvent(event)
+
 
 class MplWidget(QtWidgets.QWidget):
 
