@@ -6,13 +6,14 @@ import os
 from collections import Counter
 
 import pytest
+import numpy as np
 
-from numpy.testing import assert_allclose
+from numpy.testing import assert_allclose, assert_equal
 
 from glue.config import colormaps
 from glue.core.message import SubsetUpdateMessage
 from glue.core import HubListener, Data
-from glue.core.roi import XRangeROI, RectangularROI
+from glue.core.roi import XRangeROI, RectangularROI, CircularROI
 from glue.core.subset import RoiSubsetState, AndState
 from glue import core
 from glue.core.component_id import ComponentID
@@ -21,6 +22,7 @@ from glue.viewers.matplotlib.qt.tests.test_data_viewer import BaseTestMatplotlib
 from glue.core.state import GlueUnSerializer
 from glue.app.qt.layer_tree_widget import LayerTreeWidget
 from glue.app.qt import GlueApplication
+from glue.tests.helpers import PYSIDE2_INSTALLED  # noqa
 
 from ..data_viewer import ScatterViewer
 
@@ -52,11 +54,11 @@ class TestScatterViewer(object):
 
         self.viewer = self.app.new_data_viewer(ScatterViewer)
 
-        self.data_collection.register_to_hub(self.hub)
-        self.viewer.register_to_hub(self.hub)
-
     def teardown_method(self, method):
         self.viewer.close()
+        self.viewer = None
+        self.app.close()
+        self.app = None
 
     def test_basic(self):
 
@@ -65,16 +67,16 @@ class TestScatterViewer(object):
         # Check defaults when we add data
         self.viewer.add_data(self.data)
 
-        assert combo_as_string(self.viewer.options_widget().ui.combosel_x_att) == 'x:y:z'
-        assert combo_as_string(self.viewer.options_widget().ui.combosel_y_att) == 'x:y:z'
+        assert combo_as_string(self.viewer.options_widget().ui.combosel_x_att) == 'Main components:x:y:z:Coordinate components:Pixel Axis 0 [x]:World 0'
+        assert combo_as_string(self.viewer.options_widget().ui.combosel_y_att) == 'Main components:x:y:z:Coordinate components:Pixel Axis 0 [x]:World 0'
 
         assert viewer_state.x_att is self.data.id['x']
-        assert viewer_state.x_min == -1.1
-        assert viewer_state.x_max == 3.4
+        assert_allclose(viewer_state.x_min, -1.1 - 0.225)
+        assert_allclose(viewer_state.x_max, 3.4 + 0.225)
 
         assert viewer_state.y_att is self.data.id['y']
-        assert viewer_state.y_min == 3.2
-        assert viewer_state.y_max == 3.5
+        assert_allclose(viewer_state.y_min, 3.2 - 0.015)
+        assert_allclose(viewer_state.y_max, 3.5 + 0.015)
 
         assert not viewer_state.x_log
         assert not viewer_state.y_log
@@ -86,12 +88,12 @@ class TestScatterViewer(object):
         viewer_state.y_att = self.data.id['z']
 
         assert viewer_state.x_att is self.data.id['x']
-        assert viewer_state.x_min == -1.1
-        assert viewer_state.x_max == 3.4
+        assert_allclose(viewer_state.x_min, -1.1 - 0.225)
+        assert_allclose(viewer_state.x_max, 3.4 + 0.225)
 
         assert viewer_state.y_att is self.data.id['z']
-        assert viewer_state.y_min == -0.5
-        assert viewer_state.y_max == 2.5
+        assert_allclose(viewer_state.y_min, -0.5 - 0.15)
+        assert_allclose(viewer_state.y_max, 2.5 + 0.15)
 
         assert not viewer_state.x_log
         assert not viewer_state.y_log
@@ -102,26 +104,26 @@ class TestScatterViewer(object):
 
         self.viewer.add_data(self.data)
 
-        assert viewer_state.x_min == -1.1
-        assert viewer_state.x_max == 3.4
+        assert_allclose(viewer_state.x_min, -1.1 - 0.225)
+        assert_allclose(viewer_state.x_max, 3.4 + 0.225)
 
         self.viewer.options_widget().button_flip_x.click()
 
-        assert viewer_state.x_min == 3.4
-        assert viewer_state.x_max == -1.1
+        assert_allclose(viewer_state.x_max, -1.1 - 0.225)
+        assert_allclose(viewer_state.x_min, 3.4 + 0.225)
 
-        assert viewer_state.y_min == 3.2
-        assert viewer_state.y_max == 3.5
+        assert_allclose(viewer_state.y_min, 3.2 - 0.015)
+        assert_allclose(viewer_state.y_max, 3.5 + 0.015)
 
         self.viewer.options_widget().button_flip_y.click()
 
-        assert viewer_state.y_min == 3.5
-        assert viewer_state.y_max == 3.2
+        assert_allclose(viewer_state.y_max, 3.2 - 0.015)
+        assert_allclose(viewer_state.y_min, 3.5 + 0.015)
 
     def test_remove_data(self):
         self.viewer.add_data(self.data)
-        assert combo_as_string(self.viewer.options_widget().ui.combosel_x_att) == 'x:y:z'
-        assert combo_as_string(self.viewer.options_widget().ui.combosel_y_att) == 'x:y:z'
+        assert combo_as_string(self.viewer.options_widget().ui.combosel_x_att) == 'Main components:x:y:z:Coordinate components:Pixel Axis 0 [x]:World 0'
+        assert combo_as_string(self.viewer.options_widget().ui.combosel_y_att) == 'Main components:x:y:z:Coordinate components:Pixel Axis 0 [x]:World 0'
         self.data_collection.remove(self.data)
         assert combo_as_string(self.viewer.options_widget().ui.combosel_x_att) == ''
         assert combo_as_string(self.viewer.options_widget().ui.combosel_y_att) == ''
@@ -137,8 +139,8 @@ class TestScatterViewer(object):
         self.data.add_component([3, 4, 1, 2], 'a')
         assert self.viewer.state.x_att is self.data.id['x']
         assert self.viewer.state.y_att is self.data.id['y']
-        assert combo_as_string(self.viewer.options_widget().ui.combosel_x_att) == 'x:y:z:a'
-        assert combo_as_string(self.viewer.options_widget().ui.combosel_y_att) == 'x:y:z:a'
+        assert combo_as_string(self.viewer.options_widget().ui.combosel_x_att) == 'Main components:x:y:z:a:Coordinate components:Pixel Axis 0 [x]:World 0'
+        assert combo_as_string(self.viewer.options_widget().ui.combosel_y_att) == 'Main components:x:y:z:a:Coordinate components:Pixel Axis 0 [x]:World 0'
 
     def test_nonnumeric_first_component(self):
         # regression test for #208. Shouldn't complain if
@@ -226,12 +228,35 @@ class TestScatterViewer(object):
         # was selected
 
         self.viewer.add_data(self.data)
-        self.viewer.state.x_att = self.data.components[0]
+        self.viewer.state.x_att = self.data.id['x']
         test = ComponentID('test')
         self.data.update_id(self.viewer.state.x_att, test)
         assert self.viewer.state.x_att is test
-        assert combo_as_string(self.viewer.options_widget().ui.combosel_x_att) == 'test:y:z'
+        assert combo_as_string(self.viewer.options_widget().ui.combosel_x_att) == 'Main components:test:y:z:Coordinate components:Pixel Axis 0 [x]:World 0'
 
+    def test_density_map(self):
+
+        self.viewer.add_data(self.data)
+        self.viewer.state.layers[0].points_mode = 'auto'
+        assert len(self.viewer.layers[0].density_artist._x) == 0
+        self.viewer.state.layers[0].points_mode = 'density'
+        assert len(self.viewer.layers[0].density_artist._x) == 4
+        self.viewer.state.layers[0].points_mode = 'markers'
+        assert len(self.viewer.layers[0].density_artist._x) == 0
+
+    def test_density_map_color(self):
+
+        # Regression test to make sure things don't crash when changing
+        # back to markers if the color mode is cmap
+
+        self.viewer.add_data(self.data)
+        self.viewer.state.layers[0].points_mode = 'density'
+        self.viewer.state.layers[0].cmap_mode = 'Linear'
+        self.viewer.state.layers[0].size_mode = 'Linear'
+        self.viewer.state.layers[0].points_mode = 'markers'
+        self.viewer.state.layers[0].points_mode = 'density'
+
+    @pytest.mark.skipif('PYSIDE2_INSTALLED')
     @pytest.mark.parametrize('protocol', [0, 1])
     def test_session_back_compat(self, protocol):
 
@@ -292,7 +317,6 @@ class TestScatterViewer(object):
         assert viewer3.state.layers[1].visible
         assert not viewer3.state.layers[2].visible
 
-
     def test_session_line_back_compat(self):
 
         # Backward-compatibility for v0.11 files in which the line and scatter
@@ -341,12 +365,12 @@ class TestScatterViewer(object):
         self.viewer.add_data(self.data_2d)
 
         assert viewer_state.x_att is self.data_2d.id['a']
-        assert viewer_state.x_min == 1
-        assert viewer_state.x_max == 4
+        assert_allclose(viewer_state.x_min, 1 - 0.15)
+        assert_allclose(viewer_state.x_max, 4 + 0.15)
 
         assert viewer_state.y_att is self.data_2d.id['b']
-        assert viewer_state.y_min == 5
-        assert viewer_state.y_max == 8
+        assert_allclose(viewer_state.y_min, 5 - 0.15)
+        assert_allclose(viewer_state.y_max, 8 + 0.15)
 
         assert self.viewer.layers[0].plot_artist.get_xdata().shape == (4,)
 
@@ -357,7 +381,7 @@ class TestScatterViewer(object):
         # messages regarding subset updates (this occurred when multiple)
         # datasets were present.
 
-        layer_tree = LayerTreeWidget()
+        layer_tree = LayerTreeWidget(session=self.session)
         layer_tree.set_checkable(False)
         layer_tree.setup(self.data_collection)
         layer_tree.bind_selection_to_edit_subset()
@@ -476,3 +500,84 @@ class TestScatterViewer(object):
         viewer = ga.viewers[0][0]
         assert viewer.state.x_att is dc[0].id['z']
         assert visible_xaxis_labels(self.viewer.axes) == ['a', 'b', 'c']
+
+    def test_enable_disable_components_combo(self):
+
+        # Regression test for a bug that caused an error when turning off pixel
+        # components from combo boxes.
+
+        self.viewer.add_data(self.data)
+
+        self.data['a'] = self.data.id['x'] + 5
+
+        self.viewer.state.x_att_helper.pixel_coord = True
+
+        self.viewer.state.x_att = self.data.pixel_component_ids[0]
+
+        self.viewer.state.x_att_helper.pixel_coord = False
+
+    def test_datetime64_support(self, tmpdir):
+
+        self.data.add_component(np.array([100, 200, 300, 400], dtype='M8[D]'), 't1')
+        self.data.add_component(np.array([200, 300, 400, 500], dtype='M8[D]'), 't2')
+        self.viewer.add_data(self.data)
+        self.viewer.state.x_att = self.data.id['t1']
+        self.viewer.state.y_att = self.data.id['y']
+
+        # Matplotlib deals with dates by converting them to the number of days
+        # since 01-01-0001, so we can check that the limits are correctly
+        # converted (and not 100 to 400)
+        assert self.viewer.axes.get_xlim() == (719248.0, 719578.0)
+        assert self.viewer.axes.get_ylim() == (3.2 - 0.015, 3.5 + 0.015)
+
+        # Apply an ROI selection in plotting coordinates
+        roi = RectangularROI(xmin=719313, xmax=719513, ymin=3, ymax=4)
+        self.viewer.apply_roi(roi)
+
+        # Check that the two middle elements are selected
+        assert_equal(self.data.subsets[0].to_mask(), [0, 1, 1, 0])
+
+        # Now do the same with the y axis
+        self.viewer.state.y_att = self.data.id['t2']
+
+        assert self.viewer.axes.get_xlim() == (719248.0, 719578.0)
+        assert self.viewer.axes.get_ylim() == (719348.0, 719678.0)
+
+        # Apply an ROI selection in plotting coordinates
+        edit = self.session.edit_subset_mode
+        edit.edit_subset = []
+        roi = CircularROI(xc=719463, yc=719563, radius=200)
+        self.viewer.apply_roi(roi)
+        assert_equal(self.data.subsets[1].to_mask(), [0, 1, 1, 1])
+
+        # Make sure that the Qt labels look ok
+        self.viewer.state.y_att = self.data.id['y']
+        options = self.viewer.options_widget().ui
+        assert options.valuetext_x_min.text() == '1970-03-27'
+        assert options.valuetext_x_max.text() == '1971-02-20'
+        assert options.valuetext_y_min.text() == '3.185'
+        assert options.valuetext_y_max.text() == '3.515'
+
+        # Make sure that we can set the xmin/xmax to a string date
+        assert_equal(self.viewer.state.x_min, np.datetime64('1970-03-27', 'D'))
+        options.valuetext_x_min.setText('1970-04-14')
+        options.valuetext_x_min.editingFinished.emit()
+        assert self.viewer.axes.get_xlim() == (719266.0, 719578.0)
+        assert_equal(self.viewer.state.x_min, np.datetime64('1970-04-14', 'D'))
+
+        # Make sure that everything works fine after saving/reloading
+        filename = tmpdir.join('test_datetime64.glu').strpath
+        self.session.application.save_session(filename)
+        with open(filename, 'r') as f:
+            session = f.read()
+        state = GlueUnSerializer.loads(session)
+        ga = state.object('__main__')
+        viewer = ga.viewers[0][0]
+        options = viewer.options_widget().ui
+
+        assert_equal(self.viewer.state.x_min, np.datetime64('1970-04-14', 'D'))
+
+        assert options.valuetext_x_min.text() == '1970-04-14'
+        assert options.valuetext_x_max.text() == '1971-02-20'
+        assert options.valuetext_y_min.text() == '3.185'
+        assert options.valuetext_y_max.text() == '3.515'
